
//import Sector20.*;
import ij.*;
import ij.io.*;
import ij.util.Tools;
import ij.plugin.*;
import ij.process.*;
import ij.gui.*;
import java.io.*;
import java.util.*;

import ij.measure.*;
import java.awt.image.*;

// This plugin reads data from Sector 20 2D image files into imageJ. It reads in all 
// detectors as 32 bit images in a stack. It flips the image so it has the 
// same orientation as appears in the scan plots from the beamline. 
// As of now the scale is set when the image is created, and it is not correct when 
// rescaled using the built-in tools. 
// DB 4/4/2008

public class Sector_20_Reader extends ImagePlus implements PlugIn {

    private BufferedReader f;
    private String directory;
    private String fileName;
    ImagePlus img;
    ImagePlus imgsc;
    int scale = 1;

    public Sector_20_Reader() {
        img=null;
        imgsc=null;   
    }

    public void run(String arg) {
	OpenDialog od = new OpenDialog("Open Sector 20 2D ...", arg);
	directory = od.getDirectory();
	fileName = od.getFileName();
	if (fileName==null)
	    return;
	IJ.showStatus("Opening: " + directory + fileName);
	S20read2D(directory,fileName);
	if (img==null) return;
	if(IJ.debugMode)IJ.log("done");
	     IJ.error("Sector 20 2D Reader", "Finished!");
	imgsc.setSlice(1);
	imgsc.show();
    } // run(String arg)

    protected float[] getLineAsArray(String line) {
         StringTokenizer tokenizer = new StringTokenizer(line," ");
         int size = tokenizer.countTokens();
         float[] values = new float[size];
         int i = 0;
         while (tokenizer.hasMoreTokens()) {
             String token = tokenizer.nextToken();
             values[i] = (new Float(token)).floatValue();
             i++;
         }
         return values;
    }
    protected void S20read2D(String directory, String fileName) {
	int starIndex=-1;
	String line = "";
	String hdrline="";
	Vector lines = new Vector();
	float xmin=0.0f, xmax=0.0f, ymin=0.0f, ymax=0.0f;
	     
	try {
	    BufferedReader f = new BufferedReader(new FileReader(directory+fileName));
	    while (starIndex!=0) { // read header, look for star
	        hdrline=line; // will contain names of columns - not using right now.
	        line=f.readLine();
	       starIndex = line.indexOf('*');
	    }
	    java.util.StringTokenizer st = new java.util.StringTokenizer(line.substring(1));
	    int nxp = Integer.parseInt(st.nextToken());
	    int nyp = Integer.parseInt(st.nextToken());
	    int nImages = Integer.parseInt(st.nextToken()); // an image for each detector. 
//	    nImages = 1; //test
	    int width = Integer.parseInt(st.nextToken());
	    int height = Integer.parseInt(st.nextToken());
 	    while ((line=f.readLine())!=null) {
	         lines.add(line);
	    }
	    f.close();
	   img = NewImage.createFloatImage(fileName,width,height,nImages,NewImage.FILL_BLACK);

	   for ( int j = 0; j<nImages; j++){ 
	       for ( int i=height-1;i>=0;i--){
		    for ( int k=0;k<width;k++){
		        float[] values =   getLineAsArray((String)lines.elementAt(i*width+k));
		        if (j==0&&i==(height-1)&&k==0) {xmin=values[0]; ymin=values[nxp];}
		        if (j==0&&i==(height-1)&&k==(width-1)) xmax=values[0];
		        if (j==0&&i==0&&k==0) 
					ymax=values[nxp];
		        img.setSlice(j+1);
		        float[] pixels = (float[]) img.getProcessor().getPixels();
		        pixels[k+width*(height-1-i)] = values[nxp+nyp+j];
		    }
	       }
	   }
// Image pixel calibration
	   Calibration cal = img.getCalibration();
	   cal.pixelWidth  = (xmax-xmin)/(width-1);
	   cal.pixelHeight = (ymax-ymin)/(height-1);
	   cal.xOrigin = -xmin/cal.pixelWidth;
	   cal.yOrigin = -ymin/cal.pixelHeight;
	   if (width <400)
	  	scale = (int) 400/width;
	   imgsc = NewImage.createFloatImage(fileName,width*scale,height*scale,nImages,NewImage.FILL_BLACK);

	   for (int i=0; i< nImages; i++){
		imgsc.setSlice(i+1);
		img.setSlice(i+1);
	 	ImageProcessor proc = img.getProcessor();
		imgsc.setProcessor(fileName+" rescaled for display",proc.resize(width*scale));
	   }

// Recal rescaled image to get origin right.
	   cal=imgsc.getCalibration();  
	   cal.xOrigin = -xmin/cal.pixelWidth;
	   cal.yOrigin = -ymin/cal.pixelHeight;
	 }    
	 catch (Exception e) {
	     IJ.error("Sector 20 2D Reader", "Error reading image: "+e.getMessage());
	     return;
	 }
	   
	return;
    } // S20read2D()

}

